#include "types.h"
#include "riscv.h"
#include "defs.h"
#include "date.h"
#include "param.h"
#include "memlayout.h"
#include "spinlock.h"
#include "proc.h"
#include "sysinfo.h"

uint64
sys_exit(void)
{
  int n;
  if(argint(0, &n) < 0)
    return -1;
  exit(n);
  return 0;  // not reached
}

uint64
sys_getpid(void)
{
  return myproc()->pid;
}

uint64
sys_fork(void)
{
  return fork();
}

uint64
sys_wait(void)
{
  uint64 p;
  if(argaddr(0, &p) < 0)
    return -1;
  return wait(p);
}

uint64
sys_sbrk(void)
{
  int addr;
  int n;

  if(argint(0, &n) < 0)
    return -1;
  addr = myproc()->sz;
  if(growproc(n) < 0)
    return -1;
  return addr;
}

uint64
sys_sleep(void)
{
  int n;
  uint ticks0;

  if(argint(0, &n) < 0)
    return -1;
  acquire(&tickslock);
  ticks0 = ticks;
  while(ticks - ticks0 < n){
    if(myproc()->killed){
      release(&tickslock);
      return -1;
    }
    sleep(&ticks, &tickslock);
  }
  release(&tickslock);
  return 0;
}

uint64
sys_kill(void)
{
  int pid;

  if(argint(0, &pid) < 0)
    return -1;
  return kill(pid);
}

// return how many clock tick interrupts have occurred
// since start.
uint64
sys_uptime(void)
{
  uint xticks;

  acquire(&tickslock);
  xticks = ticks;
  release(&tickslock);
  return xticks;
}

extern char* syscallName[];

uint64
sys_trace(void)
{
  uint32 mask;
  struct proc *p;

  argint(0,(int*)&mask);
  p = myproc();
  p->tracemask = mask;

  int count = 0;
  while (count++ < 22){
    mask>>=1;
    if(mask&1){
      printf("syscall %s will be tracing\n",syscallName[count]);
    }
  }
  return 0;
}

extern int kgetfreemem(void);
extern uint64 getprocnum();
uint64
sys_info(void){
  struct proc *p = myproc();
  struct sysinfo sinfo = {kgetfreemem(),getprocnum()};
  uint64 addr;

  argaddr(0,&addr);

  if(copyout(p->pagetable, addr, (char *)&sinfo, sizeof(sinfo))<0){
    return -1;
  }
  return 0;
}
